/*****************************************************************************
*   Function:  main ()                                                       *
*                                                                            *
*   Purpose:   1) Display Cyrix CPU revision                                 *
*              2) Display Cyrix FPU detection                                *
*                                                                            *
*   Parameters:   Nothing                                                    *
*   Returns:      Nothing                                                    *
*   Calls:                                                                   *
*              cpu_type ()    - Returns int containing CPU identification    *
*              isfpu()        - 1 if fpu is present, 0 if not                *
*              iscyrixfpu()   - 1 if Cyrix fpu, 0 if not                     *
*****************************************************************************/            
            
#define TRUE      1
#define FALSE     0

// Cyrix Device ID Register 0 (DIR0) Values
#define UNKNOWN      0xff
#define Cx486_SLC    0x0      
#define Cx486_DLC    0x1      
#define Cx486_SLC2   0x2      
#define Cx486_DLC2   0x3      
#define Cx486_SRx    0x4      // Retail Upgrade Cx486SLC
#define Cx486_DRx    0x5      // Retail Upgrade Cx486DLC
#define Cx486_SRx2   0x6      // Retail Upgrade 2x Cx486SLC
#define Cx486_DRx2   0x7      // Retail Upgrade 2x Cx486DLC
#define Cx486S       0x10     // B step
#define Cx486S2      0x11     // B step
#define Cx486Se      0x12     // B step
#define Cx486S2e     0x13     // B step
#define Cx486DX      0x1a     // 
#define Cx486DX2     0x1b     // 
#define M1           0x30     
#define Cx486_pr     0xfd     // Today's Cx486SLC and DLC, No ID Register
#define Cx486S_a     0xfe     // A step, No ID Register

// Prototypes
extern int isfpu ();
unsigned short cpu_type();
int iscyrixfpu ();

main ()
   {
   unsigned short x;
    
   switch ( (x=cpu_type()) & 0xff)
      {
      case (UNKNOWN):
         printf ("\n       UNKNOWN   ");
         break;
      case (Cx486_SLC):
         printf ("\n       Cx486_SLC   ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486_DLC):
         printf ("\n       Cx486_DLC   ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;

      case (Cx486_SLC2):
         printf ("\n       Cx486_SLC2  ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;

      case (Cx486_DLC2):
         printf ("\n       Cx486_DLC2  ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;

      case (Cx486_SRx):
         printf ("\n       Cx486_SRx   ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486_DRx):
         printf ("\n       Cx486_DRx   ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486_SRx2):
         printf ("\n       Cx486_SRx2  ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486_DRx2):
         printf ("\n       Cx486_DRx2  ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486S):
         printf ("\n       Cx486S     ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486S2):
         printf ("\n       Cx486S2    ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486Se):
         printf ("\n       Cx486Se     ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486S2e):
         printf ("\n       Cx486S2e    ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486DX):
         printf ("\n       Cx486DX     ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486DX2):
         printf ("\n       Cx486DX2    ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;

      case (Cx486_pr):
         printf ("\n       Cx486SLC or DLC pre DIR regs     ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      case (Cx486S_a):
         printf ("\n       Cx486S_a     ");
         printf ("Step=1");
         break;

      case (M1):
         printf ("\n       M1    ");
         printf ("Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      default:
         printf ("\n Unkown ID");
         printf ("\n Device Type = %x",(x & 0xff));
         printf ("\n Step=%x, Rev=%x", ((x & 0xf000) >> 12),
                                                ((x &  0xf00) >>  8) );
         break;
      }

   if (isfpu())
      {
      if (iscyrixfpu ())
         printf ("   Cyrix Coprocessor");
      else
         printf ("   Other Coprocessor");
      }
   else
      printf ("   No Coprocessor");

   }
